using System;
using System.Collections;
using System.Collections.Specialized;
using System.Xml;

namespace AspNetGallery.Extensions.Modules.QueryString
{
    public class QueryStringHelper
    {
        protected static QueryStringErrorCodes ErrorCode = QueryStringErrorCodes.NoError;
        protected static Hashtable TypedValues = new Hashtable();

        public const String QueryStringValidationStatus = "QueryStringValidationStatus";
        public const String QueryStringValues = "QueryStringValues";

        public static Hashtable LoadFromFile(String fileName) 
        {
            var qsTable = new Hashtable();

            // Load the document and get the root node
            var doc = new XmlDocument();
            doc.Load(fileName);
            var root = doc.DocumentElement;

            // Visit the <page> nodes
            var pages = doc.SelectNodes("querystring/page");
            for (var i=0; i<pages.Count; i++)
            {
                var page = pages[i];
                var url = page.Attributes["url"].Value.ToLower();
                var abortOnError = true;
                var attrib = page.Attributes["abortOnError"];
                if (attrib != null)
                {
                    var pv = attrib.Value;
                    if (String.Equals(pv, "false", StringComparison.OrdinalIgnoreCase))
                        abortOnError = false;
                }

                // Create the descriptor object
                var qsDesc = new QueryStringDescriptor {Url = url, AbortOnError = abortOnError};

                // Get the object to collect param info objects
                var qsParamColl = new QueryStringParamCollection();
                qsDesc.Parameters = qsParamColl;

                // Extract the parameters information
                var paramNodeList = page.SelectNodes("param");
                for(var j=0; j<paramNodeList.Count; j++)
                {
                    var qsParamInfo = new QueryStringParamInfo();
                    var paramNode = paramNodeList[j];

                    // Extract the name of the n.th parameter
                    qsParamInfo.Name = String.Empty;
                    var paramAttrib = paramNode.Attributes["name"];
                    if (paramAttrib != null)
                        qsParamInfo.Name = paramAttrib.Value;
                
                    // Extract the optional length of the n.th parameter
                    qsParamInfo.Length = -1;
                    paramAttrib = paramNode.Attributes["length"];
                    if (paramAttrib != null)
                        qsParamInfo.Length = Convert.ToInt32(paramAttrib.Value);

                    // Extract the type of the n.th parameter
                    qsParamInfo.Type = QueryStringParamTypes.Text;
                    paramAttrib = paramNode.Attributes["type"];
                    if (paramAttrib != null)
                        qsParamInfo.Type = (QueryStringParamTypes)Enum.Parse(typeof(QueryStringParamTypes), paramAttrib.Value);
                
                    // Extract the optional status of the n.th parameter
                    qsParamInfo.Optional = false;
                    paramAttrib = paramNode.Attributes["optional"];
                    if (paramAttrib != null)
                    {
                        var pv = paramAttrib.Value;
                        if (String.Equals(pv, "true", StringComparison.OrdinalIgnoreCase))
                            qsParamInfo.Optional = true;
                    }

                    // Extract the case-sensitivity mode of the n.th parameter
                    qsParamInfo.CaseSensitive = false;
                    paramAttrib = paramNode.Attributes["casesensitive"];
                    if (paramAttrib != null) 
                    {
                        var pv = paramAttrib.Value;
                        if (String.Equals(pv, "true", StringComparison.OrdinalIgnoreCase))
                            qsParamInfo.CaseSensitive = true;
                    }

                    // Add parameter information to the collection
                    qsParamColl.Add(qsParamInfo);
                }

                // Add the descriptor to the global hash table
                qsTable.Add(url, qsDesc);
            }

            return qsTable;
        }

        public static QueryStringErrorCodes GetErrorCode()
        {
            return ErrorCode;
        }

        public static Boolean Validate(NameValueCollection postedQueryString, QueryStringDescriptor supportedQueryString)
        {
            ErrorCode = QueryStringErrorCodes.NoError;
            TypedValues.Clear();

            // Step #1 :: Check the length of the query string
            //            Some of qs parameters may be optional; so it is possible 
            //            that a valid query string is shorter than the number of 
            //            defined parameters
            if (postedQueryString.Count > supportedQueryString.Parameters.Count)
            {
                ErrorCode = ErrorCode & QueryStringErrorCodes.TooManyParameters;
                return false;
            }

            // Step #2 :: Iterate on posted parameters and ensure that each
            //            matches an entry in the supported query string 
            for (var i=0; i<postedQueryString.Count; i++)
            {
                var name = postedQueryString.Keys[i];
                if (!supportedQueryString.Parameters.Contains(name))
                {
                    ErrorCode = ErrorCode & QueryStringErrorCodes.InvalidQueryParameter;
                    return false;
                }
            }

            // Step #3 :: Iterate on supported parameters and ensure that 
            //               a) mandatory parameters have a compatible value
            //               b) optional parameters, if present, have a compatible value 
            for (var i=0; i<supportedQueryString.Parameters.Count; i++)
            {
                var paramInfo = supportedQueryString.Parameters[i];
                var name = paramInfo.Name;
                var postedValue = postedQueryString[name];

                // Check if the posted query string lack a mandatory parameter
                if (!paramInfo.Optional)
                {
                    if (postedValue == null)
                    {
                        ErrorCode = ErrorCode | QueryStringErrorCodes.MissingRequiredParameter;
                        return false;
                    }

                    if (!CheckParamValue(paramInfo, postedValue))
                    {
                        ErrorCode = ErrorCode | QueryStringErrorCodes.InvalidContent;
                        return false;   
                    }
                }
                else
                {
                    if (postedValue != null)
                    {
                        if (!CheckParamValue(paramInfo, postedValue))
                        {
                            ErrorCode = ErrorCode | QueryStringErrorCodes.InvalidContent;
                            return false;
                        }
                    }
                }
            }
            
            return true;
        }

        public static Boolean CheckParamValue(QueryStringParamInfo paramInfo, String paramValue)
        {
            QueryStringParamTypes paramType = paramInfo.Type;

            // paramValue is never NULL at this point


            // Type is Text
            // Check the length of the string matches the specified length
            if (paramType == QueryStringParamTypes.Text)
            {
                if (paramInfo.Length > -1)
                    if (paramValue.Length > paramInfo.Length)
                        return false;
            }

            // Type is Int
            if (paramType == QueryStringParamTypes.Int)
            {
                Int32 result;
                var done = Int32.TryParse(paramValue, out result);
                if (!done)
                    return false;
            }

            // Type is Bool
            if (paramType == QueryStringParamTypes.Bool)
            {
                if (!(String.Equals(paramValue, "yes", StringComparison.OrdinalIgnoreCase) ||
                      String.Equals(paramValue, "no", StringComparison.OrdinalIgnoreCase) ||
                      String.Equals(paramValue, "true", StringComparison.OrdinalIgnoreCase) || 
                      String.Equals(paramValue, "false", StringComparison.OrdinalIgnoreCase)))
                    return false;
            }

            return true;
        }

        public static Hashtable GetTypedValues(NameValueCollection postedQueryString, QueryStringDescriptor supportedQueryString)
        {
            // If here all parameters on the posted query string are OK
            var typedValues = new Hashtable();

            for (var i = 0; i < postedQueryString.Count; i++)
            {
                var name = postedQueryString.Keys[i];
                var paramInfo = supportedQueryString.Parameters.Find(name);
                if (paramInfo == null)
                    return null;

                if (paramInfo.Type == QueryStringParamTypes.Text)
                {
                    var prmValue = postedQueryString[i];
                    typedValues.Add(name, prmValue);
                }

                if (paramInfo.Type == QueryStringParamTypes.Int)
                {
                    var prmValue = GetParameterValueAsInt(postedQueryString[i]);
                    typedValues.Add(name, prmValue);
                }

                if (paramInfo.Type == QueryStringParamTypes.Bool)
                {
                    var prmValue = GetParameterValueAsBool(postedQueryString[i]);
                    typedValues.Add(name, prmValue);
                }
            }

            return typedValues;
        }

        protected static Int32 GetParameterValueAsInt(String paramValue)
        {
            var done = false;
            var result = 0;
            done = Int32.TryParse(paramValue, out result);
            if (done)
                return result;

            // Never get here, though
            return -1;
        }

        protected static Boolean GetParameterValueAsBool(String paramValue)
        {
            if (String.Equals(paramValue, "yes", StringComparison.OrdinalIgnoreCase) ||
                String.Equals(paramValue, "true", StringComparison.OrdinalIgnoreCase))
            {
                return true;
            }
            
            if (String.Equals(paramValue, "no", StringComparison.OrdinalIgnoreCase) ||
                String.Equals(paramValue, "false", StringComparison.OrdinalIgnoreCase))
            {
                return false;
            }

            // Never get here, though
            return false;
        }
    }
}