using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using Entities;

namespace LiveQuoteManager.Services
{
    public class YahooQuoteProvider : IQuoteProvider
    {
        #region Local variables
        private const string UrlBase = "http://download.finance.yahoo.com/d/quotes.csv?s={0}&f=sl1d1t1c1ohgvj1pp2owern&d=t";
        #endregion

        /// <summary>
        /// Indicates the name of the financial info provider
        /// </summary>
        public String ProviderName
        {
            get { return "Finance Yahoo"; }
        }

        #region IQuoteProvider Members

        public IList<StockInfo> GetQuotes(String symbols)
        {
            return this.FindQuoteInfo(symbols);
        }

        #endregion

        #region Helpers

        /// <summary>
        /// Gets quotes and related change using the Finance Yahoo web service
        /// </summary>
        /// <param name="symbols">Comma-separated string indicating symbols to retrieve</param>
        /// <returns>Array of StockInfo objects</returns>
        private StockInfo[] FindQuoteInfo(String symbols)
        {
            // Set the URL to invoke
            var url = String.Format(UrlBase, symbols);

            // Prepare to work
            var output = String.Empty;
            StreamReader reader = null;

            // Connect and get response
            var request = WebRequest.Create(url);
            var response = request.GetResponse();

            // Read the response
            using (reader = new StreamReader(response.GetResponseStream()))
            {
                output = reader.ReadToEnd();
                reader.Close();
            }

            // Parse and return
            return YahooQuoteProvider.ParseResponseToArray(output, this.ProviderName);
        }

        /// <summary>
        /// Parse the response from Finance Yahoo to an array of custom objects
        /// </summary>
        /// <param name="response">Raw response as received from Yahoo</param>
        /// <param name="providerName">Name to store as the data provider</param>
        /// <returns>Array of StockInfo objects</returns>
        private static StockInfo[] ParseResponseToArray(string response, string providerName)
        {
            // Prepare the container
            var data = new List<StockInfo>();

            // Split response into rows
            var rows = response.Split(new string[] { "\n" },
                StringSplitOptions.RemoveEmptyEntries);

            for (var i = 0; i < rows.Length; i++)
            {
                var tokens = rows[i].Split(',');

                // Extract symbol information
                var stock = new StockInfo
                                {
                                    Company = YahooQuoteProvider.RemoveQuotes(tokens[16]),
                                    CurrentQuote = tokens[1],
                                    Date = YahooQuoteProvider.RemoveQuotes(tokens[2]),
                                    Time = YahooQuoteProvider.RemoveQuotes(tokens[3]),
                                    Change = YahooQuoteProvider.RemoveQuotes(tokens[11])
                                };

                data.Add(stock);
            }

            return data.ToArray();
        }


        /// <summary>
        /// Remove leading and trailing double quotes from a text string 
        /// </summary>
        /// <param name="originalText">Text to process</param>
        /// <returns>String without delimiting quotes</returns>
        private static string RemoveQuotes(string originalText)
        {
            originalText = originalText.TrimEnd(new char[] { '\r', '\"' });
            return originalText.Substring(1, originalText.Length - 1);
        }
        #endregion

    }
}